###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

import os
import sys
import logging

from biolib.common import check_file_exists, make_sure_path_exists
from biolib.external.execute import check_dependencies

from genometk.metadata_nucleotide import MetadataNucleotide
from genometk.metadata_genes import MetadataGenes
from genometk.ssu import SSU


class OptionsParser():
    def __init__(self):
        """Initialization"""
        self.logger = logging.getLogger()

    def _read_config_file(self):
        """Read configuration info."""

        cfg_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), '..', 'genometk.cfg')

        d = {}
        for line in open(cfg_file):
            key, value = line.split('=')
            d[key] = value.strip()

        return d

    def nucleotide(self, options):
        self.logger.info('Calculating nucleotide properties of genome.')

        check_file_exists(options.genome_file)
        make_sure_path_exists(options.output_dir)

        meta_nuc = MetadataNucleotide()
        metadata_values, metadata_desc = meta_nuc.generate(options.genome_file,
                                                           options.contig_break)

        # write statistics to file
        output_file = os.path.join(options.output_dir, 'metadata.genome_nt.tsv')
        fout = open(output_file, 'w')
        for field in sorted(metadata_values.keys()):
            fout.write('%s\t%s\n' % (field, str(metadata_values[field])))
        fout.close()

        # write description to file
        output_file = os.path.join(options.output_dir, 'metadata.genome_nt.desc.tsv')
        fout = open(output_file, 'w')
        for field in sorted(metadata_desc.keys()):
            fout.write('%s\t%s\t%s\n' % (field,
                                         metadata_desc[field],
                                         type(metadata_values[field]).__name__.upper()))
        fout.close()

    def gene(self, options):
        self.logger.info('Calculating gene properties of genome.')

        check_file_exists(options.genome_file)
        check_file_exists(options.gff_file)
        make_sure_path_exists(options.output_dir)

        meta_genes = MetadataGenes()
        metadata_values, metadata_desc = meta_genes.generate(options.genome_file,
                                                                options.gff_file)

        # write statistics to file
        output_file = os.path.join(options.output_dir, 'metadata.genome_gene.tsv')
        fout = open(output_file, 'w')
        for field in sorted(metadata_values.keys()):
            fout.write('%s\t%s\n' % (field, str(metadata_values[field])))
        fout.close()

        # write description to file
        output_file = os.path.join(options.output_dir, 'metadata.genome_gene.desc.tsv')
        fout = open(output_file, 'w')
        for field in sorted(metadata_desc.keys()):
            fout.write('%s\t%s\t%s\n' % (field,
                                         metadata_desc[field],
                                         type(metadata_values[field]).__name__.upper()))
        fout.close()

    def ssu(self, options):
        self.logger.info('Identifying, extracting, and classifying 16S rRNA genes.')

        check_file_exists(options.genome_file)
        check_file_exists(options.ssu_taxonomy_file)
        make_sure_path_exists(options.output_dir)

        ssu = SSU(options.cpus)
        ssu.run(options.genome_file,
                    options.evalue,
                    options.concatenate,
                    options.ssu_db,
                    options.ssu_taxonomy_file,
                    options.output_dir)

    def parse_options(self, options):
        """Parse user options and call the correct pipeline(s)"""

        if options.subparser_name == 'nucleotide':
            self.nucleotide(options)
        elif options.subparser_name == 'gene':
            self.gene(options)
        elif options.subparser_name == 'ssu':
            check_dependencies(['blastn', 'nhmmer'])
            self.ssu(options)
        else:
            self.logger.error('  [Error] Unknown RefineM command: ' + options.subparser_name + '\n')
            sys.exit()

        self.logger.info('Done.')

        return 0



###############################################################################
#
# ssuFinder.py - identify SSU (16S rRNA) in genome bins
#
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

import os
import logging

import biolib.seq_io as seq_io
from biolib.external.blast import Blast
from biolib.taxonomy import Taxonomy


class SSU(object):
    """Identify, extract, and classify 16S rRNA genes."""

    def __init__(self, cpus):
        """Initialization."""
        self.logger = logging.getLogger()

        self.cpus = cpus

        file_dir = os.path.dirname(os.path.realpath(__file__))
        hmm_dir = os.path.join(file_dir, 'data_files', 'hmms')
        self.bac_ssu_model = os.path.join(hmm_dir, 'SSU_bacteria.hmm')
        self.ar_ssu_model = os.path.join(hmm_dir, 'SSU_archaea.hmm')
        self.euk_ssu_model = os.path.join(hmm_dir, 'SSU_euk.hmm')

    def _hmm_search(self, seq_file, evalue, output_dir):
        """Identify 16S rRNA genes.

        Parameters
        ----------
        seq_file : str
            File with nucleotide sequences in fasta format.
        evalue : float
            E-value threshold for defining valid hits.
        output_dir : str
            Output directory.
        """

        if seq_file.endswith('gz'):
            pipe = 'zcat ' + seq_file + ' | '
        else:
            pipe = 'cat ' + seq_file + ' | '

        output_prefix = os.path.join(output_dir, 'ssu')

        self.logger.info('Identifying bacterial 16S.')
        os.system(pipe + 'nhmmer --noali --cpu %d -o %s.hmm_bacteria.txt -E %s %s -' % (self.cpus, output_prefix, str(evalue), self.bac_ssu_model))

        self.logger.info('Identifying archaeal 16S.')
        os.system(pipe + 'nhmmer --noali --cpu %d -o %s.hmm_archaea.txt -E %s %s -' % (self.cpus, output_prefix, str(evalue), self.ar_ssu_model))

        self.logger.info('Identifying eukaryotic 18S.')
        os.system(pipe + 'nhmmer --noali --cpu %d -o %s.hmm_euk.txt -E %s %s -' % (self.cpus, output_prefix, str(evalue), self.euk_ssu_model))

    def _read_hits(self, results_file, domain, evalue_threshold):
        """Parse hits from nhmmer output.

        Parameters
        ----------
        results_file : str
            Output file from nhmmer to parse.
        domain : str
            Domain of HMM model used.
        evalue_threshold : float
            E-value threshold for defining valid hits.

        Returns
        -------
        dict : d[seq_id] -> information about hit
            Information about hits for individual sequences.
        """

        seq_info = {}

        read_hit = False
        for line in open(results_file):
            if line[0:2] == '>>':
                line_split = line.split()
                seq_id = line_split[1]
                read_hit = True
                hit_counter = 0
            elif line.strip() == '':
                read_hit = False
            elif read_hit:
                hit_counter += 1
                if hit_counter >= 3:
                    line_split = line.split()

                    iEvalue = line_split[3]
                    ali_from = int(line_split[7])
                    ali_to = int(line_split[8])

                    rev_comp = False
                    if ali_from > ali_to:
                        rev_comp = True
                        ali_from, ali_to = ali_to, ali_from

                    align_len = int(ali_to) - int(ali_from)

                    if float(iEvalue) <= evalue_threshold:
                        seq_info[seq_id] = seq_info.get(seq_id, []) + [[domain, iEvalue, str(ali_from), str(ali_to), str(align_len), str(rev_comp)]]

        return seq_info

    def _add_hit(self, hits, seq_id, info, concatenate_threshold):
        """Add hits from individual HMMs and concatenate nearby hits.

        Parameters
        ----------
        hits : d[seq_id] -> information about hit
            Information about hits for individual sequences.
        seq_id : str
            Sequence identifier with hit to add.
        info : list
            Information about hit.
        concatenate_threshold : int
            Concatenate hits within the specified number of base pairs.
        """

        # check if this is the first hit to this sequence
        if seq_id not in hits:
            hits[seq_id] = info
            return

        # check if hits to sequence are close enough to warrant concatenating them,
        # otherwise record both hits
        base_seq_id = seq_id
        index = 1
        bConcatenate = False
        concate_seq_id = seq_id
        while(True):
            # if hits overlap then retain only the longest
            start_new = int(info[2])
            end_new = int(info[3])
            rev_new = bool(info[5])

            start = int(hits[seq_id][2])
            end = int(hits[seq_id][3])
            rev = bool(info[5])

            # check if hits should be concatenated
            if abs(start - end_new) < concatenate_threshold and rev_new == rev:
                # new hit closely preceded old hit and is on same strand
                del hits[seq_id]
                info[2] = str(start_new)
                info[3] = str(end)
                info[4] = str(end - start_new)
                hits[concate_seq_id] = info
                bConcatenate = True

            elif abs(start_new - end) < concatenate_threshold and rev_new == rev:
                # new hit closely follows old hit and is on same strand
                del hits[seq_id]
                info[2] = str(start)
                info[3] = str(end_new)
                info[4] = str(end_new - start)
                hits[concate_seq_id] = info
                bConcatenate = True

            index += 1
            new_seq_id = base_seq_id + '-#' + str(index)
            if bConcatenate:
                if new_seq_id in hits:
                    seq_id = new_seq_id  # see if other sequences concatenate
                else:
                    break
            else:
                # hits are not close enough to concatenate
                if new_seq_id in hits:
                    seq_id = new_seq_id  # see if the new hit overlaps with this
                    concate_seq_id = new_seq_id
                else:
                    hits[new_seq_id] = info
                    break

    def _add_domain_hit(self, hits, seq_id, info):
        """Add hits from different domain models and concatenate nearby hits.

        Parameters
        ----------
        hits : d[seq_id] -> information about hit
            Information about hits for individual sequences.
        seq_id : str
            Sequence identifier with hit to add.
        info : list
            Information about hit.
        """

        if seq_id not in hits:
            hits[seq_id] = info
            return

        base_seq_id = seq_id
        overlap_seq_id = seq_id

        index = 1
        bOverlap = False
        while(True):
            # if hits overlap then retain only the longest
            start_new = int(info[2])
            end_new = int(info[3])
            length_new = int(info[4])

            start = int(hits[seq_id][2])
            end = int(hits[seq_id][3])
            length = int(hits[seq_id][4])

            if (start_new <= start and end_new >= start) or (start <= start_new and end >= start_new):
                bOverlap = True

                if length_new > length:
                    hits[overlap_seq_id] = info
                else:
                    hits[overlap_seq_id] = hits[seq_id]

                if overlap_seq_id != seq_id:
                    del hits[seq_id]

            index += 1
            new_seq_id = base_seq_id + '-#' + str(index)
            if new_seq_id in hits:
                seq_id = new_seq_id  # see if the new hit overlaps with this
                if not bOverlap:
                    overlap_seq_id = seq_id
            else:
                break

        if not bOverlap:
            hits[new_seq_id] = info

    def _identify_ssu(self, genome_file, evalue_threshold, concatenate_threshold, output_dir):
        """Identify 16S rRNA genes.

        Parameters
        ----------
        genome_file : str
            Name of fasta file containing nucleotide sequences.
        evalue_threshold : float
            E-value threshold for defining valid hits.
        concatenate_threshold : int
            Concatenate hits within the specified number of base pairs.
        output_dir : str
            Output directory.

        Returns
        -------
        dict : d[seq_id] -> information about best hit
            Information about best hits.
        """

        # identify 16S reads from contigs/scaffolds
        self._hmm_search(genome_file, evalue_threshold, output_dir)

        # read HMM hits
        hits_per_domain = {}
        for domain in ['archaea', 'bacteria', 'euk']:
            seq_info = self._read_hits(os.path.join(output_dir, 'ssu' + '.hmm_' + domain + '.txt'), domain, evalue_threshold)

            hits = {}
            if len(seq_info) > 0:
                for seq_id, seq_hits in seq_info.iteritems():
                    for hit in seq_hits:
                        self._add_hit(hits, seq_id, hit, concatenate_threshold)

            hits_per_domain[domain] = hits

        # find best domain hit for each sequence
        best_hits = {}
        for _, hits in hits_per_domain.iteritems():
            for seq_id, info in hits.iteritems():
                if '-#' in seq_id:
                    seq_id = seq_id[0:seq_id.rfind('-#')]

                self._add_domain_hit(best_hits, seq_id, info)

        return best_hits

    def _extract_ssu(self, genome_file, best_hits, output_dir):
        """Extract 16S rRNA genes.

        Parameters
        ----------
        genome_file : str
            Name of fasta file containing nucleotide sequences.
        best_hits : d[seq_id] -> information about best hit
            Information about best hits.
        output_dir : str
            Output directory.

        Returns
        -------
        str
            Name of fasta file containing extractracted sequences.
        """

        # write summary file and putative SSU rRNAs to file
        summary_file = os.path.join(output_dir, 'ssu.hmm_summary.tsv')
        summary_out = open(summary_file, 'w')
        summary_out.write('Sequence Id\tHMM\ti-Evalue\tStart hit\tEnd hit\tSSU gene length\tReverse Complement\tSequence length\n')

        ssu_seq_file = os.path.join(output_dir, 'ssu.fna')
        seq_out = open(ssu_seq_file, 'w')

        seqs = seq_io.read(genome_file)

        for seq_id in best_hits:
            orig_seq_id = seq_id
            if '-#' in seq_id:
                seq_id = seq_id[0:seq_id.rfind('-#')]

            seq_info = [orig_seq_id] + best_hits[orig_seq_id]
            seq = seqs[seq_id]
            summary_out.write('\t'.join(seq_info) + '\t' + str(len(seq)) + '\n')

            seq_out.write('>' + seq_info[0] + '\n')
            seq_out.write(seq[int(seq_info[3]) + 1:int(seq_info[4]) + 1] + '\n')

        summary_out.close()
        seq_out.close()

        return ssu_seq_file

    def _classify_ssu(self, seq_file, ssu_db, ssu_taxonomy_file, evalue_threshold, output_dir):
        """Classify 16S rRNA genes.

        Parameters
        ----------
        seq_file : str
            Name of fasta file containing 16S rRNA sequences.
        ssu_db : str
            BLAST database of 16S rRNA genes.
        ssu_taxonomy_file : str
            Taxonomy file for genes in the 16S rRNA database.
        evalue_threshold : float
            E-value threshold for defining valid hits.
        output_dir : str
            Output directory.
        """

        # blast sequences against 16S database
        blast = Blast(self.cpus)
        blast_file = os.path.join(output_dir, 'ssu.blastn.tsv')
        blast.blastn(seq_file, ssu_db, blast_file, evalue=evalue_threshold, max_matches=1, output_fmt='custom')

        # read taxonomy file
        taxonomy = Taxonomy().read(ssu_taxonomy_file)

        # write out classification file
        classification_file = os.path.join(output_dir, 'ssu.taxonomy.tsv')
        fout = open(classification_file, 'w')
        fout.write('query_id\tssu_taxonomy\tssu_length\tssu_blast_subject_id\tssu_blast_evalue\tssu_blast_bitscore\tssu_blast_align_len\tssu_blast_perc_identity\n')

        processed_query_ids = set()
        for line in open(blast_file):
            line_split = [x.strip() for x in line.split('\t')]
            query_id = line_split[0]

            if query_id in processed_query_ids:
                # A query may have multiple hits to different sections
                # of a gene. Blast results are organized by e-value so
                # here only the first hit is considered. The subject gene
                # is the same in all cases so the taxonomy string will
                # be identical.
                continue

            processed_query_ids.add(query_id)
            query_len = int(line_split[1])
            subject_id = line_split[2]
            align_len = line_split[5]
            perc_identity = line_split[6]
            evalue = line_split[7]
            bitscore = line_split[8]

            taxonomy_str = ';'.join(taxonomy[subject_id])

            fout.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (query_id, taxonomy_str, query_len, subject_id, evalue, bitscore, align_len, perc_identity))

        fout.close()

    def run(self, genome_file, evalue_threshold, concatenate_threshold, ssu_db, ssu_taxonomy_file, output_dir):
        """Identify, extract, and classify 16S rRNA genes.

        Parameters
        ----------
        genome_file : str
            Name of fasta file containing nucleotide sequences.
        evalue_threshold : float
            E-value threshold for defining valid hits.
        concatenate_threshold : int
            Concatenate hits within the specified number of base pairs.
        ssu_db : str
            BLAST database of 16S rRNA genes.
        ssu_taxonomy_file : str
            Taxonomy file for genes in the 16S rRNA database.
        output_dir : str
            Output directory.
        """

        self.logger.info('Identifying SSU rRNAs on sequences.')
        best_hits = self._identify_ssu(genome_file, evalue_threshold, concatenate_threshold, output_dir)
        self.logger.info('Identified ' + str(len(best_hits)) + ' putative SSU genes.')

        if len(best_hits):
            self.logger.info('Extracting SSU rRNAs on sequences.')
            ssu_seq_file = self._extract_ssu(genome_file, best_hits, output_dir)

            self.logger.info('Classifying SSU rRNA sequences.')
            self._classify_ssu(ssu_seq_file, ssu_db, ssu_taxonomy_file, evalue_threshold, output_dir)



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

import logging

import biolib.seq_io as seq_io

from numpy import (zeros as np_zeros,
                   sum as np_sum)


class GenericFeatureParser():
    """Parses generic feature file (GFF)."""
    def __init__(self, filename):
        self.cds_count = 0
        self.tRNA_count = 0
        self.rRNA_count = 0
        self.rRNA_16S_count = 0
        self.ncRNA_count = 0

        self.genes = {}
        self.last_coding_base = {}

        self._parse(filename)

        self.coding_mask = {}
        for seq_id in self.genes:
            self.coding_mask[seq_id] = self._coding_mask(seq_id)

    def _parse(self, gff_file):
        """Parse GFF file.

        Parameters
        ----------
        gff_file : str
            Generic feature file to parse.
        """

        for line in open(gff_file):
            if line[0] == '#':
                continue

            line_split = line.split('\t')
            if line_split[2] == 'tRNA':
                self.tRNA_count += 1
            elif line_split[2] == 'rRNA':
                self.rRNA_count += 1

                if 'product=16S ribosomal RNA' in line_split[8]:
                    self.rRNA_16S_count += 1
            elif line_split[2] == 'ncRNA':
                self.ncRNA_count += 1
            elif line_split[2] == 'CDS':
                self.cds_count += 1

                seq_id = line_split[0]
                if seq_id not in self.genes:
                    gene_count = 0
                    self.genes[seq_id] = {}
                    self.last_coding_base[seq_id] = 0

                gene_id = seq_id + '_' + str(gene_count)
                gene_count += 1

                start = int(line_split[3])
                end = int(line_split[4])

                self.genes[seq_id][gene_id] = [start, end]
                self.last_coding_base[seq_id] = max(self.last_coding_base[seq_id], end)

    def _coding_mask(self, seq_id):
        """Build mask indicating which bases in a sequences are coding."""

        # safe way to calculate coding bases as it accounts
        # for the potential of overlapping genes
        coding_mask = np_zeros(self.last_coding_base[seq_id])
        for pos in self.genes[seq_id].values():
            coding_mask[pos[0]:pos[1] + 1] = 1

        return coding_mask

    def coding_bases(self, seq_id):
        """Calculate number of coding bases in sequence."""

        # check if sequence has any genes
        if seq_id not in self.genes:
            return 0

        return np_sum(self.coding_mask[seq_id])

    def total_coding_bases(self):
        """Calculate total number of coding bases.

        Returns
        -------
        int
            Number of coding bases.
        """

        coding_bases = 0
        for seq_id in self.genes:
            coding_bases += self.coding_bases(seq_id)

        return int(coding_bases)


class MetadataGenes():
    """Calculate metadata derived from called genes."""

    def __init__(self):
        """Initialization"""
        self.logger = logging.getLogger()

    def generate(self, genome_file, gff_file):
        """Derive metdata from gene sequences.

        Parameters
        ----------
        genome_file : str
            Name of fasta file containing nucleotide sequences.
        gff_file : str
            Name of generic feature file describing genes.

        Returns
        -------
        dict : d[metadata_field] -> value
            Map of metadata fields to their respective values.
        dict : d[metadata_field -> description
            Description of each metadata field.
        """

        gff_parser = GenericFeatureParser(gff_file)
        coding_bases = gff_parser.total_coding_bases()

        # calculate nucleotide statistics
        scaffolds = seq_io.read(genome_file)
        genome_size = sum([len(x) for x in scaffolds.values()])

        gene_stats = {}
        gene_desc = {}

        gene_stats['protein_count'] = gff_parser.cds_count
        gene_desc['protein_count'] = "Number of protein coding genes."

        gene_stats['coding_bases'] = coding_bases
        gene_desc['coding_bases'] = "Number of coding bases in genome."

        gene_stats['coding_density'] = float(coding_bases) * 100.0 / genome_size
        gene_desc['coding_density'] = "Percentage of coding bases in genome."

        # This quantities are not being reported here as they are only specified
        # in the NCBI GFF files. A general GFF file (e.g., for prodigal) only
        # specifies identified CDS.
        # gene_stats['tRNA_count'] = gff_parser.tRNA_count
        # gene_desc['tRNA_count'] = "Number of tRNA genes identified in genome."
        # gene_stats['ncRNA_count'] = gff_parser.ncRNA_count
        # gene_desc['ncRNA_count'] = "Number of ncRNA genes identified in genome."
        # gene_stats['rRNA_count'] = gff_parser.rRNA_count
        # gene_desc['rRNA_count'] = "Number of rRNA genes identified in genome."
        # gene_stats['16S_count'] = gff_parser.rRNA_16S_count
        # gene_desc['16S_count'] = "Number of 16S rRNA genes identified in genome."

        return gene_stats, gene_desc



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

import logging

import biolib.seq_io as seq_io
import biolib.seq_tk as seq_tk
import biolib.genome_tk as genome_tk


class MetadataNucleotide():
    """Calculate metadata derived from nucleotide sequences."""

    def __init__(self):
        """Initialization"""
        self.logger = logging.getLogger()

    def generate(self, genome_file, contig_break):
        """Derive metdata across nucleotide sequences.

        Parameters
        ----------
        genome_file : str
            Name of fasta file containing nucleotide sequences.
        contig_break : int
            Minimum number of ambiguous bases for defining contigs.

        Returns
        -------
        dict : d[metadata_field] -> value
            Map of metadata fields to their respective values.
        dict : d[metadata_field -> description
            Description of each metadata field.
        """

        # calculate nucleotide statistics
        scaffolds = seq_io.read(genome_file)

        nuc_stats = {}
        nuc_desc = {}

        nuc_stats['scaffold_count'] = len(scaffolds)
        nuc_desc['scaffold_count'] = "Number of scaffolds in genome."
        nuc_stats['gc_count'] = genome_tk.gc_count(scaffolds)
        nuc_desc['gc_count'] = "Number of G or C bases in genome."
        nuc_stats['gc_percentage'] = genome_tk.gc(scaffolds) * 100.0
        nuc_desc['gc_percentage'] = "GC content of genome."
        nuc_stats['genome_size'] = sum([len(x) for x in scaffolds.values()])
        nuc_desc['genome_size'] = "Total base pairs in genome including nucleotide bases, ambiguous bases, and gaps."
        nuc_stats['n50_scaffolds'] = seq_tk.N50(scaffolds)
        nuc_desc['n50_scaffolds'] = "Scaffold length at which 50% of total bases in assembly are in scaffolds of that length or greater."
        nuc_stats['l50_scaffolds'] = seq_tk.L50(scaffolds, nuc_stats['n50_scaffolds'])
        nuc_desc['l50_scaffolds'] = "Number of scaffolds longer than, or equal to, the scaffold N50 length."
        nuc_stats['mean_scaffold_length'] = int(seq_tk.mean_length(scaffolds))
        nuc_desc['mean_scaffold_length'] = "Mean length of scaffolds in base pairs."
        nuc_stats['longest_scaffold'] = seq_tk.max_length(scaffolds)
        nuc_desc['longest_scaffold'] = "Number of bases in longest scaffold."

        contigs = seq_tk.identify_contigs(scaffolds, 'N' * contig_break)
        nuc_stats['contig_count'] = len(contigs)
        nuc_desc['contig_count'] = "Number of contigs in genome."
        nuc_stats['ambiguous_bases'] = genome_tk.ambiguous_nucleotides(contigs)
        nuc_desc['ambiguous_bases'] = "Number of ambiguous bases in contigs."
        nuc_stats['total_gap_length'] = genome_tk.ambiguous_nucleotides(scaffolds) - nuc_stats['ambiguous_bases']
        nuc_desc['total_gap_length'] = "Number of ambiguous bases comprising gaps in scaffolds."
        nuc_stats['n50_contigs'] = seq_tk.N50(contigs)
        nuc_desc['n50_contigs'] = "Contig length at which 50% of total bases in assembly are in contigs of that length or greater."
        nuc_stats['l50_contigs'] = seq_tk.L50(contigs, nuc_stats['n50_contigs'])
        nuc_desc['l50_contigs'] = "Number of contigs longer than, or equal to, the contig N50 length."
        nuc_stats['mean_contig_length'] = int(seq_tk.mean_length(contigs))
        nuc_desc['mean_contig_length'] = "Mean length of contigs in base pairs."
        nuc_stats['longest_contig'] = seq_tk.max_length(contigs)
        nuc_desc['longest_contig'] = "Number of bases in longest contig."

        return nuc_stats, nuc_desc



from distutils.core import setup

import os


def version():
    setupDir = os.path.dirname(os.path.realpath(__file__))
    versionFile = open(os.path.join(setupDir, 'genometk', 'VERSION'))
    return versionFile.read().strip()

setup(
    name='GenomeTk',
    version=version(),
    author='Donovan Parks',
    author_email='donovan.parks@gmail.com',
    packages=['genometk'],
    scripts=['bin/genometk'],
    package_data={'genometk' : ['VERSION', 'data_files/hmms/*']},
    url='http://pypi.python.org/pypi/genometk/',
    license='GPL3',
    description='A toolbox for working with genomes.',
    long_description=open('README.md').read(),
    install_requires=[
        "numpy >= 1.8.0",
        "biolib >= 0.0.11"],
)
